#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 46elks AB <hello@46elks.com>
# Developed in 2015 by Emil Tullstedt <emil@46elks.com>
# Licensed under the MIT License

"""
elkme is a commandline utility to send sms from the terminal
"""

from __future__ import print_function
from .config import read_config, generate_config, default_config_location
from .elks import Elks
import argparse
import os
import sys
import json
from .helpers import b, s

ELK = """
  ,;MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM;,.
/MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM.
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM`   `^'  `Q/^^\\MMMpcqMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM;,             `^'   `VP    YP'  `MM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMDomm;,._                     /M
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMP`        _.,,=rRMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMP^^^^MMMM'          MMMMMMMMMMMMMMMMMM
MMMMMMMMP`              '^^oMMMPP`       ``'            \\MMMMMMMMMMMMMMMM
MMMMMMM'                                                  ``\\MMMMMMMMMMMM
MMMMMM'                                                          'QMMMMMM
MMM/`                                                              \\MMMMM
MM/_=o,                                     ,/     \\_               `MMMM
MMMMMMM,                                   /MM     pMM\\,._           MMMM
MMMMMMMM,                                 ,MMMM, pMMMMMMMX          /MMMM
MMMMMMMMP                                 AMMMMMMMMMMMMMMMM\\m____.pMMMMMM
MMMMMMMP                                  MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMM|         ,.mPDMMMMMMMMpo.,        \\MMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMM|      ,mPMMMMMMMMMMMMMMMMMDo       \\MMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMM|    /MMMMMMMMMMMMMMMMMMMMMMMM\\       \\MMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMM|    MMMMMMMMMMMMMMMMMMMMMMMMMMDp,.    `MMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMP     MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMP    \\MMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMM|    `MMMMMMMMMMMMMMMMMMMMMMMMMMMMMM|     \\MMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMPPDmmMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMBYMMJOHANNESLMMOFMM46ELKSMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
\\MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMP
 `\\MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM`
"""

HELPTEXT = """
Send a text message using the commandline powered by the 46elks
API available from https://www.46elks.com.
textme also supports pipes
"""

USAGE = """Hello,
elkme is a tool for sending text messages using the commandline.

To use elkme, you need to have an account on 46elks at https://46elks.com and
login to your dashboard to receive your API username and password.

Run `elkme "Hello internet" -u APIUSERNAME -p APIPASSWORD -t +46700000000` to
send a text message from commandline. That's it. (oh, and don't forget to
add --saveconf so that you don't have to enter the settings every time you
run the application)

See `elkme --help` for more information about elkme"""


def main():
    """Executed on run"""

    try:
        input = raw_input
    except NameError:
        pass

    args = parse_args()

    if args.configfile:
        conffile = os.path.expanduser(args.configfile)
    else:
        conffile = default_config_location()

    conf = read_config(conffile)

    if args.quiet < 1:
        conf['verbose'] = True
    if args.verbose >= 1 and args.quiet < 1:
        conf['debug'] = True
    if args.to:
        conf['to'] = args.to
    if args.sender:
        conf['from'] = args.sender
    if args.username:
        conf['username'] = args.username
    if args.password:
        conf['password'] = args.password

    message = None
    if args.message and not args.file:
        message = args.message
    elif args.file:
        with open(args.file, 'r') as openfile:
            message = openfile.read()
    elif not sys.stdin.isatty():
        message = ''
        try:
            while True:
                message += input()
                message += '\n'
        except EOFError:
            pass

    if args.saveconf:
        try:
            with open(conffile, 'w') as fdest:
                settings = generate_config(conf)
                settings.write(fdest)
        except IOError as e:
            print(e)
        if not message:
            exit(0)

    invalid_conf = False
    try:
      elks_conn = Elks(conf)
    except KeyError:
      invalid_conf = True

    if args.me:
        elks_conn.list_user()
        exit(0)
    if args.numbers:
        if not message:
          message = ''
        numbers = elks_conn.list_numbers(all = 'all' in message)
        for i, number in enumerate(numbers):
          print("(%s) %s" % (i, number))
        exit(0)

    if not message:
        print(USAGE, file=sys.stderr)
        exit(-1)

    if args.verbose >= 2 or (message[0:3][0] == 'elks'):
        print(ELK)

    if invalid_conf:
      print(USAGE, file=sys.stderr)
      print("\n\nYour configuration is invalid", file=sys.stderr)
      exit(-1)

    if args.call:
        response = elks_conn.make_call(message,
            conf.get('to', None),
            conf.get('from', None))
        if 'debug' in conf:
            print(s(response))
        elif 'verbose' in conf:
            retval = json.loads(s(response))
            print('Made connection to ' + conf['to'])
    else:
        send_sms(elks_conn, conf, message)

def parse_args():
    """Parse the arguments to the application"""
    parser = argparse.ArgumentParser(
        description=HELPTEXT,
        epilog="This application is powered by elks with superpowers!")
    parser.add_argument('-v', '--verbose', action='count',
                        help="Debug output", default=0)
    parser.add_argument('-q', '--quiet', action='count',
                        help="Suppress most output", default=0)
    parser.add_argument('message', metavar='message', type=str, nargs='*',
                        help="The message to be sent (<160 characters)")
    parser.add_argument('-f', '--file', metavar='file', action='store',
                        help="""File to read message from (only the
                        first 160 characters are sent)""")
    parser.add_argument('-t', '--to', dest='to', action='store',
                        help="Phone number to receive the text message")
    parser.add_argument('-s', '--sender', '--from', dest='sender',
                        action='store', help="""
                        Sender of the message. See 46elks' API documentation
                        for valid formats""")
    parser.add_argument('-u', '--username', dest='username', action='store',
                        help="Your API username from https://www.46elks.com/")
    parser.add_argument('-p', '--password', dest='password', action='store',
                        help="Your API password from https://www.46elks.com/")
    parser.add_argument('--saveconf', dest='saveconf',
                        action='count', help="""
                        Generates a configuration file from the commandline
                        options and exits.""")
    parser.add_argument('-c', '--config', dest='configfile',
                        help="""Location of the custom configuration file""")
    parser.add_argument('--call', '--dial', action='store_true', default=False,
                        help="""Make a call""")
    parser.add_argument('--me', action='store_true', help="User info")
    parser.add_argument('--numbers', action='store_true',
                        help="Show my numbers")
    return parser.parse_args()

def send_sms(conn, conf, message):
    sender = conf.get('from', 'elkme')
    to = conf.get('to', None)

    response = conn.send_sms(message[:159], to, sender)

    if 'debug' in conf:
        print(s(response))
    elif 'verbose' in conf:
        retval = json.loads(s(response))

        if len(message) > 160:
            print(message)
            print('----')
            print('Sent first 160 characters to ' + retval['to'])
        else:
            print('Sent "' + retval['message'] + '" to ' + retval['to'])

if __name__ == '__main__':
    main()




#!/usr/bin/env python

import sys
import json

if sys.version_info < (3,):
    def b(x):
        return x

    def s(x):
        return x
else:
    def b(x):
        return bytes(x, 'utf-8')

    def s(x):
        return x.decode('utf-8')


def parse_payload(payload):
    if not isinstance(payload, str):
        payload = ' '.join(payload)

    try:
        json.loads(payload)
    except ValueError:
        kv = payload.split(' ', 1)
        if len(kv) > 1:
          payload = '{"%s": "%s"}' % (kv[0], kv[1])
        else:
          payload = '%s' % kv[0]

    return payload




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 46elks AB <hello@46elks.com>
# Developed in 2015 by Emil Tullstedt <emil@46elks.com>
# Licensed under the MIT License

from __future__ import print_function
from base64 import b64encode
try:
    from urllib import urlencode
    from urllib2 import HTTPError, urlopen, Request
except ImportError:
    from urllib.parse import urlencode
    from urllib.error import HTTPError
    from urllib.request import urlopen, Request
import json
import sys
from .helpers import b, s, parse_payload

class Elks:
    username = None
    password = None
    api_url = "https://api.46elks.com/a1/%s"

    def __init__(self, conf={}):
        default_base_url = 'https://api.46elks.com/a1'
        self.username = conf.get('username', None)
        self.password = conf.get('password', None)
        self.api_url = '%s/' % (conf.get('api_url', default_base_url)) + '%s'

    def query_api(self, data=None, endpoint='SMS'):
        url = self.api_url % endpoint
        if data:
            conn = Request(url, b(urlencode(data)))
        else:
            conn = Request(url)

        auth = b('Basic ') + b64encode(b(self.username + ':' + self.password))
        conn.add_header('Authorization', auth)

        try:
            response = urlopen(conn)
        except HTTPError as err:
            print(err)
            print("\nSending didn't succeed :(")
            exit(-2)
        return response.read()


    def validate_number(self, number):
        if not isinstance(number, str):
            raise Exception('Phone number may not be empty')
        if number[0] == '+' and len(number) > 2 and len(number) < 16:
            return True
        else:
            raise Exception("Phone number must be of format +CCCXXX...")


    def format_sms_payload(self, message, to, sender='elkme', options=[]):
        self.validate_number(to)

        if not isinstance(message, str):
            message = " ".join(message)

        message = message.rstrip()

        sms = {
            'from': sender,
            'to': to,
            'message': message
        }
        
        for option in options:
          if option not in ['dontlog', 'dryrun', 'flash']:
            raise Exception('Option %s not supported' % option)
          sms[option] = 'yes'

        return sms

    def send_sms(self, message, to, sender='elkme', options=[]):
        """Sends a text message to a configuration conf containing the message
        in the message paramter"""
        sms = self.format_sms_payload(message=message,
            to=to,
            sender=sender,
            options=options)
        return self.query_api(sms)

    def format_call_payload(self, payload, to, sender):
        voice_start = parse_payload(payload)

        call = {
            'from': sender,
            'to': to,
            'voice_start': voice_start
        }
        return call

    def make_call(self, payload, to, sender):
        call = self.format_call_payload(payload, to, sender)
        return self.query_api(call, 'Calls')

    def list_user(self):
        response = self.query_api(endpoint='Me')
        return json.loads(s(response))

    def list_numbers(self, all = False):
        response = self.query_api(endpoint='Numbers')
        numbers = json.loads(s(response))['data']
        if not all:
            numbers = filter(lambda num: num['active'] == 'yes', numbers)
        numbers = map(lambda num: num['number'], numbers)
        return list(numbers)




#!/usr/bin/env python

from setuptools import setup, find_packages

setup(name='elkme',
      version='0.4.1',
      description='Command-line tool and SDK for sending SMS',
      long_description="""elkme is a command-line tool for sending SMS using
      the 46elks platfrom from the terminal which doubles as an SDK to use
      when building applications utilizing the 46elks platform""",
      url="https://github.com/46elks/elkme",
      author="Emil Tullstedt",
      author_email='emil@46elks.com',
      license='MIT',
      packages = find_packages(),
      packages_dir = {'': 'elkme'},
      classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries',
        'Topic :: System :: Monitoring'
      ],
      keywords="sms 46elks cli monitoring",
      entry_points={
            'console_scripts': [
                'elkme = elkme.main:main'
                ]
          },
      use_2to3 = False,
      test_suite = 'nose.collector')




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 46elks AB <hello@46elks.com>
# Developed in 2015 by Emil Tullstedt <emil@46elks.com>
# Licensed under the MIT License

from __future__ import print_function
try:
    import ConfigParser
except ImportError:
    import configparser as ConfigParser
import sys
import os
import platform


def default_config_location(Filename="elkme"):
    home = os.path.expanduser('~')
    location = home + os.sep + "." + Filename

    if platform.system() == "Darwin":
        path = home + os.sep + "Library" + os.sep + "Application Support"\
                + os.sep
        location = path + Filename
    elif platform.system() == "Linux":
        path = home + os.sep + ".config" + os.sep
        location = path + Filename
        if not os.path.isdir(path):
            os.mkdir(path)
    elif platform.system() == "Windows":
        # Might break on Windows <= XP
        # That's ok, since XP is no longer supported by MSFT
        location = os.environ["LOCALAPPDATA"] + os.sep + Filename + ".ini"

    # Migration if textme-conffile exists but not elkme
    if Filename == "elkme" and not os.path.isfile(location):
        oldconf = default_config_location("textme")
        if os.path.isfile(oldconf):
            os.rename(oldconf, location)
    return location


def read_config(path, section="46elks"):
    """Reads configuration from a configuration file using the
    ConfigParser library"""
    config = ConfigParser.RawConfigParser()
    try:
        config.read(path)
    except ConfigParser.MissingSectionHeaderError:
        return {}
    settings = {}
    try:
        for element in config.items(section):
            settings[element[0]] = element[1]
    except ConfigParser.NoSectionError:
        return {}
    return settings


def generate_config(conf, section="46elks"):
    """
    Generates a configuration file using the ConfigParser library that
    can be saved to a file for subsequent reads
    """
    config = ConfigParser.RawConfigParser()
    config.add_section(section)
    if 'username' in conf:
        config.set(section, 'username', conf['username'])
    if 'password' in conf:
        config.set(section, "password", conf['password'])
    if 'to' in conf:
        config.set(section, "to", conf['to'])
    if 'from' in conf:
        config.set(section, "from", conf['from'])
    if not config.items(section):
        error = "You need to provide options to be stored as"
        error += " commandline options"
        print(error, file=sys.stderr)
    if 'verbose' in conf:
        print("Wrote to the config file :)")
    return config

